package com.ehsure.ehsurescanner;

import android.Manifest;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.View;
import android.view.Window;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import io.flutter.embedding.android.FlutterActivity;
import me.dm7.barcodescanner.zbar.Result;
import me.dm7.barcodescanner.zbar.ZBarScannerView;

public class ScannerActivity extends FlutterActivity implements ZBarScannerView.ResultHandler {

    private ZBarScannerView zBarScannerView;
    private ImageView ivFlashlight;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_er_scanner);
        zBarScannerView = findViewById(R.id.view_ZBarScanner);
        ivFlashlight = findViewById(R.id.iv_flashlight);
        TextView tv_input = findViewById(R.id.tv_input);
        tv_input.setOnClickListener(v -> showInputAlertDialog(this::setResultAndFinish));
        // if the device does not have flashlight in its camera,
        // then remove the switch flashlight button...
        if (!hasFlash()) {
            ivFlashlight.setVisibility(View.GONE);
        } else {
            ivFlashlight.setOnClickListener((v -> {
                boolean flash = zBarScannerView.getFlash();
                if (flash) {
                    ivFlashlight.setImageResource(R.mipmap.ic_ope_flashlight);
                } else {
                    ivFlashlight.setImageResource(R.mipmap.ic_flashlight_on);
                }
                zBarScannerView.setFlash(!flash);
            }));
        }
        requestPermission();
    }

    private void requestPermission() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            List<String> permissionList = new ArrayList<>();
            permissionList.add(Manifest.permission.CAMERA);
            int checkCameraPermission = ContextCompat.checkSelfPermission(this, Manifest.permission.CAMERA);
            if (checkCameraPermission != PackageManager.PERMISSION_GRANTED) {
                ActivityCompat.requestPermissions(this, permissionList.toArray(new String[0]), 629);
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (grantResults.length == 0) {
            return;
        }
        if (requestCode == 629) {
            if (grantResults[0] == PackageManager.PERMISSION_DENIED) {
                Toast.makeText(this,"很遗憾你把权限禁用了,部分功能将无法使用!",Toast.LENGTH_SHORT).show();
            }
        } else {
            super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    private void setResultAndFinish(String result){
        Intent intent = new Intent();
        intent.putExtra("result", result);
        setResult(RESULT_OK, intent);
        finish();
    }

    @Override
    public void onResume() {
        super.onResume();
        zBarScannerView.setResultHandler(this);
        zBarScannerView.startCamera();
    }

    @Override
    public void onPause() {
        super.onPause();
        zBarScannerView.stopCamera();
    }

    /**
     * Check if the device's camera has a Flashlight.
     *
     * @return true if there is Flashlight, otherwise false.
     */
    private boolean hasFlash() {
        return getApplicationContext().getPackageManager()
                .hasSystemFeature(PackageManager.FEATURE_CAMERA_FLASH);
    }


    private void showInputAlertDialog(final OnConfirmListener listener) {
        AlertDialog.Builder builder;
        builder = new AlertDialog.Builder(this);
        builder.setTitle("请输入条码值");
        final EditText editText = new EditText(this);
//        editText.setFilters(new InputFilter[] { new InputFilter.LengthFilter(inputLength) }); // 最大输入长度
//        editText.setInputType(InputType.TYPE_CLASS_NUMBER);
        editText.setSingleLine(true);
        editText.setGravity(Gravity.CENTER);
        editText.setTextColor(Color.BLACK);
        Timer timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                InputMethodManager imm = (InputMethodManager) getSystemService(Activity.INPUT_METHOD_SERVICE);
                imm.toggleSoftInput(0, InputMethodManager.HIDE_NOT_ALWAYS);
            }
        }, 500);
        builder.setView(editText);
        builder.setPositiveButton("确定", null);
        builder.setNegativeButton("取消", (dialog, which) -> {
            dialog.dismiss();
            hideSoftInput(editText);
        });
        final AlertDialog dialog = builder.create();
        dialog.show();
        dialog.getButton(AlertDialog.BUTTON_POSITIVE).setOnClickListener(v -> {
            String input = editText.getText().toString();
            if (!TextUtils.isEmpty(input)) {
                listener.confirmCallback(input);
                dialog.dismiss();
                hideSoftInput(editText);
            } else
                Toast.makeText(this,"请输入条码值",Toast.LENGTH_SHORT).show();
        });
    }

    @Override
    public void handleResult(Result rawResult) {
        String result = rawResult.getContents();
        setResultAndFinish(result);
    }

    private interface OnConfirmListener {
        void confirmCallback(String text);
    }

    private void hideSoftInput(EditText edit) {
        edit.clearFocus();
        InputMethodManager inputManger = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        inputManger.hideSoftInputFromWindow(edit.getWindowToken(), 0);
    }

}
