import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class EhsureScanner {
  static const MethodChannel _channel = const MethodChannel('ehsure_scanner');

  static Future<String> getScanResult (BuildContext context) async {
    if (Platform.isIOS) {
      String scanResult = await Navigator.push(context, MaterialPageRoute(builder: (context) {
        return _IosScanView();
      }));
      return scanResult;
    } else {
      final String scanValue = await _channel.invokeMethod('scan');
      print(scanValue);
      return scanValue;
    }
  }
}

class _IosScanView extends StatefulWidget {
  @override
  __IosScanViewState createState() => __IosScanViewState();
}

class __IosScanViewState extends State<_IosScanView> {
  @override
  void initState() {
    super.initState();
    handleChannel();
  }
  handleChannel() {
    final channel = MethodChannel("ehsure_scanner");
    channel.setMethodCallHandler((handler) {
      if (handler.method == "scanResult") {
        Navigator.of(context).pop(handler.arguments.toString());
      }
      return null;
    });
  }

  //弹出服务器地址框
  Future<String> showInputDialog() {
    String inputString; //服务器地址
    return showDialog<String>(
        context: context,
        builder: (context) {
          return AlertDialog(
            title: Text("手动输入"),
            content: Column(
              mainAxisSize: MainAxisSize.min,
              mainAxisAlignment: MainAxisAlignment.center,
              children: <Widget>[
                Row(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    Expanded(
                      child: TextField(
                        decoration: InputDecoration(hintText: ""),
                        onChanged: (value) {
                          inputString = value;
                        },
                        keyboardType: TextInputType.url,
                      ),
                    )
                  ],
                ),
              ],
            ),
            actions: <Widget>[
              FlatButton(
                child: Text('取消'),
                onPressed: () {
                  Navigator.of(context).pop();
                },
              ),
              FlatButton(
                child: Text('确定'),
                onPressed: () {
                  Navigator.of(context).pop(inputString);
                },
              )
            ],
          );
        });
  }
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text("二维码扫描")),
      body: Container(
        child: Stack(
          fit: StackFit.expand,
          children: <Widget>[
            UiKitView(
              viewType: 'ehsure_scanner_plugin',
              gestureRecognizers: <Factory<OneSequenceGestureRecognizer>>[
                new Factory<OneSequenceGestureRecognizer>(
                      () => new EagerGestureRecognizer(),
                ),
              ].toSet(),
            ),
            Positioned(
              left: 0,
              right: 0,
              top: 100,
              height: 50,
              child: Center(child: FlatButton(
                onPressed: () async {
                   String result = await showInputDialog();
                   if (result != null) {
                     Navigator.of(context).pop(result);
                   }
                },
                child: Text("手动输入", style: TextStyle(color: Colors.white),),)
              ),
            )
          ],
        ),
      ),
    );
  }
}
